--------------------------------------------------
-- Create Date:    11:52:01 06/06/2012 
--------------------------------------------------
library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

-----------------------------------------------------------------------------------------------------------
--1. frequency of ramp
--2. mean level for ramp
--3. peak detected
--4. reference FBG number and sensor FBG number
-----------------------------------------------------------------------------------------------------------
entity display_freq_peak is
    Port (
        clk          : in   STD_LOGIC;
        reset        : in   STD_LOGIC;
        sw           : in   STD_LOGIC;
        sw1          : in   STD_LOGIC; 
        sw2          : in   STD_LOGIC;  
        count_fbgref : in   integer range 0 to MAX_FBGS; 
        count_fbgsen : in   integer range 0 to MAX_FBGS;
        count_freq   : in   integer range 0 to MAX_COUNT;
        count_mean   : in   integer range 0 to 100; 
        peak         : in   STD_LOGIC_VECTOR (RAMP_BITS-1 downto 0);
        anode        : out  STD_LOGIC_VECTOR (3 downto 0);
        display      : out  STD_LOGIC_VECTOR (7 downto 0));
end display_freq_peak;

architecture Behavioral of display_freq_peak is

    constant erro   : std_logic_vector(RAMP_BITS-1 downto 0) := (others => '1'); 

    signal num_freq : std_logic_vector(3 downto 0); 
    signal num_mean : std_logic_vector(7 downto 0); 
    signal num_bcd  : std_logic_vector(NUMBCD_BITS-1 downto 0); 
    signal num_peak : std_logic_vector(NUMBCD_BITS-1 downto 0); 

    type state_type is (idle, led0, led1, led2, led3); 
    signal state_reg , state_next : state_type; 
    signal num_reg, num_next      : std_logic_vector (4 downto 0); -- BCD codes
    signal anode_reg, anode_next  : std_logic_vector(3 downto 0); 
    signal count_reg, count_next  : unsigned(17 downto 0); -- frequency of leds
    signal clk_scan               : std_logic; -- together with count_reg to control leds
    signal num_fbgref             : std_logic_vector(3 downto 0); 
    signal num_fbgsen             : std_logic_vector(3 downto 0); 

begin
    con_binary_bcd_unit: entity work.con_binary_bcd(behavioral)
        port map(clk => clk, peak => peak, num_bcd => num_bcd); 
   
    num_peak <= "1110101010100000" when (peak = erro) else
               num_bcd; 
               
    with count_fbgref select
        num_fbgref <= 
                    "0000"       when 0, 
                    "0001"       when 1, 
                    "0010"       when 2, 
                    "0011"       when 3, 
                    "0000"       when others; 
                    
    with count_fbgsen select
        num_fbgsen <= 
                    "0000"       when 0, 
                    "0001"       when 1, 
                    "0010"       when 2, 
                    "0011"       when 3, 
                    "0000"       when others;                 
    
    with count_mean select
        num_mean <= 
                    "00000000"  when 0,  --Nvel mdio: 0.0V
                    "00000110"  when 6,  --Nvel mdio: 0.6V
                    "00010010"  when 12,  --Nvel mdio: 1.2V
                    "00011000"  when 18,  --Nvel mdio: 1.8V
                    "00100100"  when 24,  --Nvel mdio: 2.4V
                    "00110000"  when 30,  --Nvel mdio: 3.0V
                    "00110110"  when 36,  --Nvel mdio: 3.6V
                    "01000010"  when 42,  --Nvel mdio: 4.2V
                    "01001000"  when 48,  --Nvel mdio: 4.8V
                    "01010100"  when 54,  --Nvel mdio: 5.4V
                    "01100000"   when 60, --Nvel mdio: 6.0V
                    "01100110"  when 66, --Nvel mdio: 6.6V
                    "01110010"  when 72, --Nvel mdio: 7.2V
                    "01111000"  when 78, --Nvel mdio: 7.8V
                    "10000100"  when 84, --Nvel mdio: 8.4V
                    "10010000"  when 90, --Nvel mdio: 9.0V
                    "10010110"  when 96, --Nvel mdio: 9.6V
                    "00000000"  when others;
        
     
    with count_freq select
        num_freq <= "0010" when 610,   --  digit 2 of 20Hz
                    "0011" when 407,   --  digit 3 of 30Hz
                    "0100" when 305,   --  digit 4 of 40Hz
                    "0101" when 244,   --  digit 5 of 50Hz
                    "0010" when others;

    with num_reg select                -- the digit that a display will show
        display <=  "00000011" when "00000",  -- 0
                    "00000010" when "00001",  -- 0.
                    "10011111" when "00010",  --1
                    "10011110" when "00011",  --1.
                    "00100101" when "00100",   --2
                    "00100100" when "00101",   --2.
                    "00001101" when "00110",  --3
                    "00001100" when "00111",  --3.
                    "10011001" when "01000",  --4
                    "10011000" when "01001",  --4.
                    "01001001" when "01010",  --5
                    "01001000" when "01011",  --5.
                    "01000001" when "01100",  --6
                    "01000000" when "01101",  --6.
                    "00011111" when "01110",  --7
                    "00011110" when "01111",  --7.
                    "00000001" when "10000",  --8
                    "00000000" when "10001",  --8.
                    "00001001" when "10010",  --9
                    "00001000" when "10011",  --9.
                    "01100001" when "11100",  --E
                    "00010001" when "10100",  --A
                    "11111111" when others;   --dark
               
anode <= anode_reg; 

-- FSMD register
process (clk, reset)
begin 
    if rising_edge(clk) then
        if reset = '1' then 
            state_reg <= idle; 
            num_reg   <= "00000";
            anode_reg <= "1111"; 
            count_reg <= (others => '0'); 
        else
            state_reg <= state_next; 
            num_reg   <= num_next; 
            anode_reg <= anode_next; 
            count_reg <= count_next; 
        end if; 
    end if;           
end process; 

count_next <= count_reg + 1; 
clk_scan <= '1' when count_reg = "11111111111111111" else '0'; 

-- FSMD next logic 
process (sw, sw1, num_freq, num_peak, num_mean, num_reg, anode_reg, clk_scan, state_reg, sw1, sw2, num_fbgref, num_fbgsen)
begin 
    state_next <= state_reg; 
    num_next   <= num_reg; 
    anode_next <= anode_reg; 
    
    case state_reg is 
        when idle =>
            if clk_scan = '1' then
                state_next <= led0; 
            end if; 
            
        when led0 =>
            if sw = '0' then         -- show the peak detected
                anode_next <= "1110"; 
                num_next <= num_peak(3 downto 0) & '0'; 
            else
                if (sw1 = '0' and sw2 = '0') then     -- show the mean level for DAC     
                    anode_next <= "1110"; 
                    num_next   <= num_mean(3 downto 0) & '0'; 
                end if; 
                
                if (sw1 = '0' and sw2 = '1') then   -- show the frequency of the ramp        
                    anode_next <= "1110"; 
                    num_next   <= "00000";
                end if; 
                
                if (sw1 = '1' and sw2 = '0') then     -- show the selected reference FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgref & '0'; 
                end if; 
                
                if (sw1 = '1' and sw2 = '1') then     -- show the selected sensor FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgsen & '0'; 
                end if; 
                
            end if; 
            
            if clk_scan = '1' then
                state_next <= led1; 
            end if; 
            
        when led1 =>
            if sw = '0' then
                anode_next <= "1101"; 
                num_next <= num_peak(7 downto 4) & '0';
            else
                if (sw1 = '0' and sw2 = '0') then     -- show the mean level for DAC       
                    anode_next <= "1101"; 
                    num_next   <= num_mean(7 downto 4) & '1'; 
                end if; 
                
                if (sw1 = '0' and sw2 = '1') then     -- show the frequency of the ramp    
                    anode_next <= "1101"; 
                    num_next   <= num_freq & '0';
                end if; 
                
                if (sw1 = '1' and sw2 = '0') then     -- show the selected reference FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgref & '0'; 
                end if; 
                
                if (sw1 = '1' and sw2 = '1') then     -- show the selected sensor FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgsen & '0'; 
                end if; 
                
            end if; 
            
            
            if clk_scan = '1' then
                state_next <= led2; 
            end if; 
            
        when led2 =>
            if sw = '0' then
                anode_next <= "1011"; 
                num_next <= num_peak(11 downto 8) & '0'; 
            else
                if (sw1 = '0' and sw2 = '0') then     -- show the mean level for DAC     
                    anode_next <= "1110"; 
                    num_next   <= num_mean(3 downto 0) & '0'; 
                end if; 
                
                if (sw1 = '0' and sw2 = '1') then     -- show the frequency of the ramp     
                    anode_next <= "1110"; 
                    num_next   <= "00000";
                end if; 
                
                if (sw1 = '1' and sw2 = '0') then     -- show the selected reference FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgref & '0'; 
                end if; 
                
                if (sw1 = '1' and sw2 = '1') then     -- show the selected sensor FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgsen & '0'; 
                end if; 
                
            end if; 
            
            
            if clk_scan = '1' then
                state_next <= led3; 
            end if; 
            
       when led3 =>
            if sw = '0' then
                anode_next <= "0111"; 
                num_next <= num_peak(15 downto 12) & '0'; 
            else 
                if (sw1 = '0' and sw2 = '0') then     -- show the frequency of the ramp     
                    anode_next <= "1101"; 
                    num_next   <= num_mean(7 downto 4) & '1'; 
                end if; 
                
                if (sw1 = '0' and sw2 = '1') then     -- show the frequency of the ramp      
                    anode_next <= "1101"; 
                    num_next   <= num_freq & '0';
                end if; 
                
                if (sw1 = '1' and sw2 = '0') then     -- show the selected reference FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgref & '0'; 
                end if; 
                
                if (sw1 = '1' and sw2 = '1') then     -- show the selected sensor FBG number   
                    anode_next <= "1110"; 
                    num_next   <= num_fbgsen & '0'; 
                end if;
                 
            end if; 
            
            if clk_scan = '1' then
                state_next <= idle; 
            end if; 
    end case; 
end process; 

end Behavioral;